% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/memoise.R
\name{forget}
\alias{forget}
\title{Forget past results.
Resets the cache of a memoised function. Use \code{\link{drop_cache}} to
reset the cache only for particular arguments.}
\usage{
forget(f)
}
\arguments{
\item{f}{memoised function}
}
\description{
Forget past results.
Resets the cache of a memoised function. Use \code{\link{drop_cache}} to
reset the cache only for particular arguments.
}
\examples{
memX <- memoise(function() { Sys.sleep(1); runif(1) })
# The forget() function
system.time(print(memX()))
system.time(print(memX()))
forget(memX)
system.time(print(memX()))
}
\seealso{
\code{\link{memoise}}, \code{\link{is.memoised}}, \code{\link{drop_cache}}
}
